<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/branches/maintain-3/class/dao/extended/cname.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |       
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Daniel Robert <robert@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: cname.php 4535 2006-06-27 18:15:56Z claym $

/**
  * cname.php - contains the {@link CNAME} class
  *
  * @author $Author: claym $
  * @filesource
  * @package    classes
  * @subpackage dao
  */

include_once DAODIR.'/Cname.php';

/**
  * A cname object
  *
  * @package    classes
  * @subpackage dao
  */
class CNAME extends DAO_Cname {

    // {{{ vars

    /** FQDN */
	var $hostname;

    // }}}

    // {{{ CNAME (constructor)

    /**
      * Constructor
      *
      * @param  int ID (primary key)
      */
	function CNAME ($cname_id = 0) 
    {
        if ($cname_id) {
            $this->id = $cname_id;
            $this->get($this->id);
        }

    } // CNAME

    // }}}

// Setters
    // {{{ setAll()
    function setAll() {
        $this->setName($this->name);
        $this->setEnabled(1);
    }
    // }}}
    // {{{ setName()
    function setName($nm = '')
    {
        if ($nm) {
            $domain = new Domain($this->domain);

            $this->name = $nm;
            $this->hostname = $this->name . '.' . $domain->name;
        }

        return true;
    }
    // }}}

// Getters
    // {{{ _toArray()
    function _toArray()
    {
        $arr = array();
        $arr['hostname'] = $this->getHostname();
        return $arr;
    }
    // }}}
    // {{{ getHostname()
    function getHostname()
    {
        if (!$this->hostname) {
            $this->setName($this->name);
        }

        return $this->hostname;
    }
    // }}}

// Validators
    // {{{ validateName()

	function validateName() 
    {
		global $conf, $user;

        if (!isset($this->domain))
            return _("No domain was set for this CNAME");

        if (!isset($this->name))
            return _("No name was set for this CNAME");

        // we'll need a domain object
        $domain = new Domain($this->domain);

		// first let's make sure it isn't in the db already
        // "SELECT * FROM cname WHERE name LIKE '$name' AND domain='$domain->id'"
        $cname = DB_DataObject::Factory('cname');
        $cname->name = $this->name;
        $cname->domain = $domain->id;
        if($this->id)
            $cname->whereAdd("id!='$this->id'");
        
        /* Check to see if the name already exists AND we're not updating ($_REQUEST[id]) */
		if ($cname->find() && !isset($this->id))
            return "$this->name.$domain->name "._("is already in use by a CNAME");

        // now we need to check to make sure there isn't a Cname with this name.
        // "SELECT * FROM host WHERE name LIKE '$name' AND domain='$domain->id'"
        $dbdo = DB_DataObject::Factory('host');
        $dbdo->domain = $this->domain;
        $dbdo->whereAdd("name LIKE '$this->name'");

        if ($dbdo->find())
            return "$this->name.$domain->name "._("is already in use by a Host");

        //now check the MX records...
        // "SELECT * FROM mx WHERE name LIKE '$name' AND domain='$domain->id'"
        $dbdo = DB_DataObject::Factory('mx');
        $dbdo->domain = $this->domain;
        $dbdo->whereAdd("name LIKE '$this->name'");

        if ($dbdo->find())
            return "$this->name.$domain->name "._("is already in use by an MX record");

		// now check the PTR records
        // "SELECT id FROM pointer WHERE hostname LIKE '$hostname' AND type='forward'"
   
        $dbdo = DB_DataObject::Factory('pointer');
        $dbdo->selectAdd();
        $dbdo->selectAdd('id');
        $dbdo->type = 'forward';
        $dbdo->whereAdd("hostname LIKE '$this->hostname'");

		if ($dbdo->find())
            return "$this->name.$domain->name "._("is already in use by a Forward Pointer");

		// next let's make sure there aren't invalid chr's in it
		//if (!(preg_match("/^[A-Za-z0-9\-]+$/", $clone->name))) {
		if (!preg_match("/^\w+(\-\w+)*$/", $this->name))
            return "$this->name "._("is an invalid CNAME");

        return true;

	}
    // }}}
    // {{{ validateServer()
    // used to take $server
    /**
     * checks the server entry for a cname
     */
    function validateServer() 
    {
        global $conf, $user;

        // break up the FQDN for $this->server
        $server = split_hostname($this->server);
		
		/*first let's see if there is a host with this record*/

		// "SELECT host.id FROM host,domain WHERE host.domain=domain.id AND domain.name='$data->domain' AND host.name='$data->name'";
 
        // set up the host dbdo
        $dbdo_host = DB_DataObject::Factory('host');
        $dbdo_host->name = $server->name;
        $dbdo_host->selectAdd();
        $dbdo_host->selectAdd('host.id');

        // set up the domain dbdo
        $dbdo_domain = DB_DataObject::Factory('domain');
        $dbdo_domain->name = $server->domain;

        // join them
        $dbdo_host->joinAdd($dbdo_domain);

		if ($dbdo_host->find())
			$host_exists = 1;
	
		/* let's make sure they aren't trying to point it at a mx record */

		// "SELECT * FROM mx,domain WHERE mx.domain=domain.id AND domain.name='$data->domain' AND mx.name='$data->name'"; 

        // set up the mx dbdo
        $dbdo_mx = DB_DataObject::Factory('mx');
        $dbdo_mx->name = $server->name;
       
        // set up the domain dbdo
        $dbdo_domain = DB_DataObject::Factory('domain');
        $dbdo_domain->name = $server->domain;

        // join them
        $dbdo_mx->joinAdd($dbdo_domain);

		if ($dbdo_mx->find() && !$host_exists) {

            $msg = "$this->server ". _("is a current MX record.").' ';
            $msg .= _("CNAMEs cannot point at MX records.");

            return $msg;

        }

        if (!Domain::is_valid_name($this->server))
            return "$this->server "._("is an invalid server name");

        return true;

    } // validateServer
    // }}}
    // {{{ validateTtl()
    /**
     * Validate the ttl field
     */
    function validateTtl() {
        global $conf;

        if (!is_numeric($this->ttl))
            return _("TTL must be numeric");

        return true;
    } // validateTtl
    // }}}
    // {{{ validateEnabled
    function validateEnabled() 
    {
        global $conf;

        if ($this->enabled !== 0 && $this->enabled !== 1)
            return false;

        return true;
    }
    // }}}

// Private Functions
    // {{{ _history_text()

    function _history_text()
    {
        // : HOSTNAME --> SERVER
        return $this->getHostname() . ' --> ' . $this->getServer();
    }

    // }}}

} //CNAME

?>
