<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL: https://svn.osuosl.org/public/maintain-project/trunk/class/htdocs/forms/HostDisplayForm.php $
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Brad Morgan <morgabra@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: $

require_once(FORMDIR."/MaintainForm.php");

class HostBulkZoneMoveForm extends MaintainForm
{

var $errors = array();

//{{{   form_init_elements()

    /**
     *  This method intializes the fields for the user
     */
    function form_init_elements()
    {
        $dbdo = DB_DataObject::Factory('zone');
        $dbdo->selectAdd();
        $dbdo->selectAdd('id');
        $dbdo->selectAdd('name');
        $dbdo->orderBy('name ASC');
        $zoneary = array();

        $dbdo->find();
        while($dbdo->fetch()) {
            $zoneary[$dbdo->name] = $dbdo->id;
        }

        $zonelist = new FEListBox(_("Target Zone"));
        $zonelist->set_list_data($zoneary);
 
        $var = $_REQUEST['hostlist'];
    
        $this->add_element(new FEHidden("action","bulk host zone move"));
        $this->add_element(new FEHidden("hostlist",serialize($var)));
        $this->add_element($zonelist);
    }
//}}}
//{{{   form()

    /**
     *  This method lays the form out for display.
     *
     *  @return DIVTag  $div    Div that contains the form
     */
    function form()
    {
        $div = html_div("form");
        $cont = container();
        $cont->add(_("You are going to move the following hosts:"),html_br());
        
        $hostlist = $_REQUEST['hostlist'];
        if(!is_array($hostlist))
            $hostlist = unserialize(stripslashes($hostlist));
        
        foreach($hostlist as $hostid) {
           $host = new Host($hostid);
           $cont->add($host->getHostname() . ' ' . $host->getIp(), html_br());
        }
        $cont->add(html_br(),(_("To zone:")));
        $cont->add($this->element_form(_('Target Zone')),html_br());    
        $cont->add(html_br());
        $cont->add(_("*NOTE: All zone specific data will be erased, and the hosts will be disabled. Remember to re-enable and update all hosts upon moving!"), html_br());
        $cont->add(html_br(),$this->add_action(_("Confirm")),$this->add_action(_("Cancel")));
        $cont->add($this->element_form("action"));
        $cont->add($this->element_form("hostlist"));
        $div->add($cont);

        return $div;
    }

//}}}
//{{{   form_action

    /**
     *  This method defines the behavior for the form based on the button pressed
     *
     *  @return boolean     true if action was successful
     */
    function form_action()
    {
        global $user;
       
        $res = true;
       
        switch(strtolower($this->get_action()))
        {
        case 'confirm':
            if(!isset($_REQUEST['hostlist']))
                break;
            $hosts = unserialize(stripslashes($_REQUEST['hostlist']));
            
            $host = new Host();
            $host->query('BEGIN');

            foreach($hosts as $hostid) {
                    
                $host = new Host($hostid);
                $targetzone = $this->get_element_value(_("Target Zone"));

                //Setting the new zone
                $oldzone = $host->zone;
                $host->setZone($targetzone);
                
                $host->setEnabled(0);
                //From here on is a bunch of checks to determine what data
                //has to be reset, and what can stay.
                
                //If the domain is not part of the zone we are moving to, then
                //this will set the domain to null
                $zone_domain = DB_DataObject::Factory('zone_domain');
                $zone_domain->zone = $targetzone;
                $zone_domain->domain = $host->domain;
        
                if(!$zone_domain->find()) {
                    $host->setDomain(0);
                }
                
                //Checks to see if the domain's workgroup is valid in the 
                //target zone, if not, it sets it to null.
                $zone_workgroup = DB_DataObject::Factory('zone_workgroup');
                $zone_workgroup->zone = $targetzone;
                $zone_workgroup->workgroup = $host->workgroup;

                if(!$zone_workgroup->find()) {
                    $host->setWorkgroup(0);
                }
 
                //If the host has a static IP addy, then this will check if
                //the range that address falls in is useable in the targetzone,
                //if not, it sets the ip adress to 0.
                if($host->ip){
                    
                    $ranges = DB_DataObject::Factory('ranges');
                    $ranges->whereAdd();
                    $ranges->whereAdd('start <= '.$host->ip);
                    $ranges->whereAdd('end >= '.$host->ip);

                    $ranges->find(true);

                    $zone_range = DB_DataObject::Factory('zone_range');
                    $zone_range->zone = $targetzone;
                    $zone_range->range = $ranges->id;

                    if(!$zone_range->find()) {
                        $host->setIp(0);  
                    }
                }
                
                //If the host has a dynamic range, it will see if that range is
                //applicable in the target zone. If not, it sets it to 0.
                if($host->dynamic_range) {
                    $zone_range = DB_DataObject::Factory('zone_range');
                    $zone_range->zone = $targetzone;
                    $zone_range->range = $host->dynamic_range;

                    if(!$zone_range->find()) {
                        $host->setDynamic_range(0);
                    }
                 }
                 
                 //If a host cannot update, we have a problem :( 
                 //So we rollback all changes so far, and add an error.
                 if(!$host->update()) {
                    $host->query('ROLLBACK');
                    $errors[$host->getName()] = _("Move failed for host : ").$host->getName();
                    break;
                 }

                 else {
                 $host->write_history(_("Moved Host from").' '.$oldzone.' '.
                 _("to").' '.$targetzone, ACTION_MODIFY, $user->username, 
                 $host->zone);
                 }
            
            }
            //If there are errors, this will display them, and return that the
            //operation failed.
            if(!empty($errors)) {
                foreach($errors as $hostid=>$error) {
                    $this->add_error("Host $hostid", $error);
                } 
                return false;
            }
            else { 
            //Otherwise, life is amazing, and we win the internet
            $host->query('COMMIT');
            return true;

            
            }
        break;

        case 'cancel':
            header("Location: host.php");
            break;
        default:
            die_hard(_("Invalid form action"));
            break;
        }
    }

//}}}
//{{{ form_backend_validation()
    function form_backend_validation()
    {
        return true;
    }
//}}}
//{{{ form_success()
    function form_success()
    {
        $div = html_div("success");
        $indxlink = html_a("index.php", "[Return to Home]");
        $hostlink = html_a("host.php", "[Return to Host List]");
        $div->add(container("Hosts moved successfully!", html_br(),html_br()));
        $div->add(container("*Note: Recently moved hosts have all non zone-specific data removed, and they are set as disabled. Remember to update recently moved hosts!", html_br()));
        $div->add($indxlink, $hostlink);
        return $div;
    }
//}}}

}

