<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 foldmethod=marker: */
// $HeadURL$
// +------------------------------------------------------------------------+
// | Maintain - DNS/DHCP Managment Tool                                     |
// +------------------------------------------------------------------------+
// | Copyright (c) 2004 Open Source University                              |
// +------------------------------------------------------------------------+
// | This program is free software; you can redistribute it and/or          |
// | modify it under the terms of the GNU General Public License            |
// | as published by the Free Software Foundation; either version 2         |
// | of the License, or (at your option) any later version.                 |
// |                                                                        |
// | This program is distributed in the hope that it will be useful,        |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of         |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          |
// | GNU General Public License for more details.                           |
// |                                                                        |
// | You should have received a copy of the GNU General Public License      |
// | along with this program; if not, write to the Free Software            |
// | Foundation, Inc., 59 Temple Place - Suite 330,                         |
// | Boston, MA  02111-1307, USA.                                           |
// +------------------------------------------------------------------------+
// | Author(s): Daniel Robert <robert@osuosl.org>                           |
// +------------------------------------------------------------------------+
// $Id: auth_ldap.php 4453 2006-06-09 20:14:56Z claym $

/**
  * auth_ldap.php - contains the {@link Auth_Ldap} class
  *
  * @author $Author$
  * @filesource
  * @package    modules
  * @subpackage auth_ldap
  */

require_once INCLUDEDIR."/auth.php";

/**
  * Authenticate users against an LDAP server
  *
  * @package    modules
  * @subpackage auth_ldap
  */
class Auth_LDAP extends AuthModule
{

    /** PEAR_Auth container for LDAP */
    var $_container;

/* Module Overloads */
// {{{ _install()

    function _install()
    {
        return true;
    }

// }}}
// {{{ _uninstall()

    function _uninstall()
    {
        return true;
    }

// }}}
// {{{ initialize()

    function initialize()
    {
        //Inserting code to sanitize the auth file until the pear people
        //decide to be nice and not === check everything... including numbers >_<
        $fixed = new Auth_LDAP_Fixer($this->config);
        
        $res = $fixed->fix_broken();
       
        //This is the way we triple equal baby
        if($res === true)
            $this->_container = new MaintAuth_Container_LDAP($fixed->getConfig());
        else
            PEAR::RaiseError("Config file could not be fixed.  LDAP Aborted");
    }

// }}}

/* AuthMod Overloads */
// {{{ auth()

    function auth($username, $password) 
    {
        global $conf;

        // we need the PEAR_Auth extended classes
        require_once CLASSDIR.'/auth.php';

        // instantiate the Auth object with the given container
        $a = new MaintAuth($this->_container, '', '', false); 
        // set the username and password (Pear::Auth demands $_POST vars by default)
        $a->setAuthVars($username, $password);

        // start the session, and attempt to authenticate the user
        $a->start();

        // if the user was authenticated, we're done
        if ($a->checkAuth()) {
            $this->_status = _("success");
            return true;
        }

        // otherwise, let's explain what went wrong
        switch($a->getStatus()) {

        case AUTH_WRONG_LOGIN:
            $this->_status = $conf['auth']['login_failed'];
            break;  

        default:
            $this->_status = $conf['auth']['bad_auth_cred'];
            break;  
        }

        return false;
    }

// }}}
// {{{ hasUser()

    function hasUser($username)
    {
        if ($this->_container->getUser($username)) {
            return true;
        }

        return false;
    }

// }}}

/* Methods for Config File Intergrity Checking */
// {{{ needsConfigFile()

    /**
      * Returns, if the module needs a config file at all. This will be used
      * by the automated config file integrity checks, for example.
      * Defaults to false.
      * Can be called statically.
      *
      * @return boolean .cfg file vital for the module to work?
      */
    function needsConfigFile()
    {
        return true;
    }
// }}}

} //Auth_LDAP class

require_once "Auth/Container/LDAP.php";

/**
  * Overloaded methods from the LDAP container in PEAR_Auth,
  * allowing us to initialize the class automatically from 
  * Maintain's config, etc.
  *
  * @package    modules
  * @subpackage auth_ldap
  */
class MaintAuth_Container_LDAP extends Auth_Container_LDAP {

    // {{{ Constructor

    /**
      * Calls parent's constructor based on ldap section
      * of the config file
      */
    function MaintAuth_Container_LDAP($config) 
    {
        parent::Auth_Container_LDAP($config);
    }

    // }}}
    //{{{ getUser()

    /**
      * Overloading an as-of-yet unwritten function in Auth_Container_LDAP, so
      * currently users Auth_Container's getUser() function.  This function
      * reads a list of all users on the server, and searches the array
      * for the given username.
      * This is a terrible method of doing it, so a direct search for a given
      * username, using LDAP search filters is used instead.
      *
      * This function as of now doesn't take advantage of any Auth_Container
      * functionality, and makes direct php ldap calls using values from
      * the config file.  This should be re-written to utilize built-in
      * values and functions in Auth_Container_LDAP
      *
      * @param  string  $username   Name of user to search for
      * @return boolean true if username was found
      */
    function getUser($username) 
    {
        /* Commented out since if the module  is enabled it's legal.
        // if we're not authenticating against LDAP, return false
        if (!valid_auth('ldap')) {
            return false;
        }
        */
        
        // what the #@!^*(# was this doing?
        /*if (preg_match("/^\w{1,4}\_.+$/",$name))
            return true;*/

        // construct the ldap uri
        if (!isset($this->options['url'])) {
            $url = 'ldap://'.$this->options['host'].':'.$this->options['port'];

            if (!ereg("^[[:alpha:]]{4,5}://[[:alnum:]][[:alnum:]-]*:[[:digit:]]{3}$", $url))
                die ("$url is not a valid LDAP uri");
        } else
            $url = $this->options['url'];

        // connect to the server
        if ($ldap_link = ldap_connect($url)) {

            // determine the protocol version
            $ver = 0;
            if (ldap_get_option($ldap_link, LDAP_OPT_PROTOCOL_VERSION, $ver) && $ver >= 2)
                ldap_set_option($ldap_link, LDAP_OPT_PROTOCOL_VERSION, 3);

            // See if the user authenticates themselves or not
            if (!isset($this->options['binddn']))
                $bind = ldap_bind($ldap_link, $this->options['binddn'], $this->options['bindpw']);

            else
                $bind = true;
           
            //now let's try to bind to the server
            if ($bind) {

                // see if the user's dn exists
                $filter = ($this->options['userattr']) ? $this->options['userattr'] : 'uid';

                $sr = ldap_search($ldap_link, $this->options['basedn'],
                        "($filter=$username)",
                        array($filter));

                $info = ldap_get_entries($ldap_link, $sr);
               
                // If it does, return true and close
                if ($info["count"]) {
                    ldap_close($ldap_link);
                    return true;
                }
                else {
                    //unable to find user
                    $error = _("The username")." $username "._("was not found, please try again.")."<br />";
                    return false;
                }

            }

            // if the user bind failed, there's not much we can do
            else { 
                $error =  $this->options['ldap_auth_dn'] .' '._(" was unable to connect to ldap server")."<br />";
                die($error._("Please contact your system administrator!"));
            }
        
        } // connect to ldap server 

        // we could not connect to the ldap server
        else {
            $error = _("Unable to connect to")." " . $this->options['ldap_host'] . "<br />";
            die($error._("Please contact your system administrator!"));
        }
        ldap_close($ldap_link);

        // we should never get here!!
        return false;

    }

    //}}}

}

class Auth_LDAP_Fixer {

    var $broken; //broken config file
    
    function Auth_LDAP_Fixer($config)
    {
        $this->broken = $config;
    }

    function fix_broken()
    {
        if(array_key_exists('debug',$this->broken))
            $this->broken['debug'] = (bool)$this->broken['debug'];
        if(array_key_exists('version',$this->broken))
            $this->broken['version'] = intval($this->broken['version']);
        return true;
    }

    function getConfig()
    {
        return $this->broken;
    }

}

?>
